# Standard lib imports
# None

# Third party imports
# None

# Project level imports
# None


class Capacity():

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_capacity(self):
        """
        Get the current managed capacity.

        Example JSON result from the API:

        """
        # TODO: Add the JSON output (endpoint when tested was returning an error, confirmed this with cURL
        return self.conn.get(url='object/capacity')



# Standard lib imports
# None

# Third party imports
# None

# Project level imports
# None


class UserInfo():

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def whoami(self):
        """
        Perform an HTTP GET against the ViPR endpoint to
        retrieve the list of tenants that the user
        maps to including the details of the mappings.
        It also returns a list of the virtual data center
        roles and tenant roles assigned to this user

        Example JSON result from the API:

        {
            "subtenant_roles": [],
            "distinguished_name": "root",
            "home_tenant_roles": [],
            "vdc_roles": [
                "RESTRICTED_SECURITY_ADMIN",
                "SYSTEM_AUDITOR",
                "RESTRICTED_SYSTEM_ADMIN",
                "SYSTEM_MONITOR"
            ],
            "common_name": "root",
            "tenant": "urn:storageos:TenantOrg:0474851a-9481-474d-a8f7-29cfaf07f536:global"
        }
        """

        return self.conn.get('user/whoami.json')

    def get_tenant(self, username=None):
        """
        Perform an HTTP GET against the ViPR endpoint to
        evaluate the tenancies that this user maps to based
        on the mappings defined in the Tenants in the system.

        If the username param is not specified it will fetch
        the common_name from the whoami results.

        Example JSON result from the API:

        {
            "tenant": [
                {
                    "id": "urn:storageos:TenantOrg:9474851b-9481-474d-a8f1-29cfaf07f536:global"
                }
            ]
        }

        :param username: The username for which to retrieve the tenant list
        """

        if username:
            params = {
                'username': username
            }
        else:
            whoami_data = self.whoami()
            params = {
                'username': whoami_data['common_name']
            }

        return self.conn.get(
            'user/tenant.json', params=params)



# Standard lib imports
# None

# Third party imports
# None

# Project level imports
# None


class Audit():
    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_audit_log(self, time_bucket=None):
        """
        Perform an HTTP GET against the ViPR endpoint to
        retrieve the bulk auditlogs and alerts in a
        specified time bucket (minute or hour)

        Example JSON result from the API:

        {
            "auditlogs": [
                {
                    "audit_type": "SSH_LOGIN",
                    "description": ""SSHauthentication(login/execution)forsvcuserfrom172.29.114.148to172.29.114.149succeed."",
                    "time_occurred": 1416355120000,
                    "operational_status": "SUCCESS",
                    "auditlog_id": "urn:storageos:AuditLog:de398a6e-2760-4b27-8440-6656355062c2",
                    "service_type": "SystemAudit",
                    "product_id": "ViPR 1.0"
                }
            ]
        }

        :param time_bucket: Time bucket for retrieval of audit logs.
        Acceptable formats are: yyyy-MM-dd'T'HH for hour bucket,
        yyyy-MM-dd'T'HH:mm for minute bucket
        Example: 2014-11-19T00:00
        """
        params = {
            'time_bucket': time_bucket
        }

        return self.conn.get('audit/logs', params=params)



# Standard lib imports
# None

# Third party imports
# None

# Project level imports
# None


class ViperpyException(Exception):
    """
    This is a custom ViperPy Exception class to better handle
    errors that ViPR can return in HTML format rather than JSON
    """

    def __init__(self, message=None, http_status_code=None, vipr_message=None):
        """
        This is a custom ViperPy Exception class to handle

        :param message: A custom
        :param http_status_code:
        :param vipr_message:
        :return:
        """
        if message is None:
            self.message = 'The ViPR endpoint has thrown an error, check ' \
                           'the http_status_code and vipr_message attributes ' \
                           'of this exception for more details.'
        else:
            self.message = message

        self.http_status_code = http_status_code
        self.vipr_message = vipr_message

        super(ViperpyException, self).__init__(
            message, http_status_code, vipr_message)



# Standard lib imports
# None

# Third party imports
# None

# Project level imports
# None


class ObjectSoftQuota():

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_bucket_quota(self, namespace, bucket_name):
        """
        Gets Bucket SoftQuota

        Required role(s):

        This call has no restrictions

        Example JSON result from the API:

        TODO: JSON data
        """
        return self.conn.get(
            url='object/softquota/buckets/{0}/{1}'.format(
                namespace, bucket_name))

    def get_namespace_quota(self, namespace):
        """
        Gets Namespace SoftQuota

        Required role(s):

        This call has no restrictions

        Example JSON result from the API:

        TODO: JSON data
        """
        return self.conn.get(
            url='object/softquota/namespace/{0}'.format(namespace))

    def update_bucket_quota(self, bucket_name, soft_quota_in_gb):
        """
        Update the Bucket Soft Quota

        Required role(s):

        This call has no restrictions

        Example JSON result from the API:

        TODO: JSON data
        """
        self._verify_is_num(soft_quota_in_gb)

        payload = {
            "namespace": "",
            "soft_quota_in_gb": soft_quota_in_gb
        }

        return self.conn.put(
            url='object/softquota/buckets/{0}'.format(bucket_name),
            json_payload=payload)

    def update_namespace_quota(self, namespace, soft_quota_in_gb):
        """
        Update the Namespace Soft Quota

        Required role(s):

        This call has no restrictions

        Example JSON result from the API:

        TODO: JSON data
        """
        self._verify_is_num(soft_quota_in_gb)

        payload = {
            "soft_quota_in_gb": soft_quota_in_gb
        }

        return self.conn.put(
            url='object/softquota/namespace/{0}'.format(namespace),
            json_payload=payload)

    def remove_bucket_quota(self, namespace, bucket_name):
        """
        Remove Bucket Quota

        Required role(s):

        This call has no restrictions

        Example JSON result from the API:

        TODO: JSON data
        """
        return self.conn.put(
            url='object/softquota/buckets/{0}/{1}'.format(
                namespace, bucket_name))

    @staticmethod
    def _verify_is_num(value):
        try:
            return int(value)
        except ValueError as err:
            raise err



# Standard lib imports
# None

# Third party imports
# None

# Project level imports
# None


class Billing():

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_bucket_billing_info(self, namespace, bucket_name):
        """
        Gets billing information about a bucket, specifically the current
        object count and storage utilization for the bucket.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        TENANT_ADMIN

        TODO: Add JSON sample

        :param namespace: The namespace containing the bucket
        :param bucket_name: The bucket name
        """
        return self.conn.get(
            'object/billing/buckets/{0}/{1}/info'.format(
                namespace, bucket_name))

    def get_bucket_namespace_info(
            self, namespace, include_bucket_detail, marker=None):
        """
        Gets billing information about a namespace and its buckets. Note that
        due to the fact that sampling a namespace's buckets takes some time
        it's possible that the values for the buckets will not sum to equal
        the namespace's values.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        TENANT_ADMIN

        TODO: Add JSON sample

        :param namespace: The namespace containing the bucket
        :param include_bucket_detail: Whether to include bucket details
        :param marker: Used to continue a truncated response. Omit this
        parameter on the first request
        """

        params = {
            'include_bucket_detail': include_bucket_detail,
            'marker': marker
        }

        return self.conn.get(
            'object/billing/namespace/{namespace}/info'.format(namespace),
            params=params)

    def get_bucket_billing_sample(
            self, namespace, bucket_name, start_time, end_time):
        """
        Gets billing information for a bucket over the given time range.
        By default, buckets are sampled every 5 minutes. If the requested
        time range includes multiple samples, the data will be aggregated.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        TENANT_ADMIN


        TODO: Add JSON sample

        :param namespace: The namespace containing the bucket
        :param bucket_name: The bucket name
        :param start_time: Starting time for the sample(s) in ISO-8601 minute
        format
        :param end_time: Ending time for the sample(s) in ISO-8601 minute
        format
        """
        params = {
            'start_time': start_time,
            'end_time': end_time
        }

        return self.conn.get(
            'object/billing/buckets/{0}/{1}/sample'.format(
                namespace, bucket_name), params=params)

    def get_namespace_billing_sample(
            self, namespace, start_time, end_time, include_bucket_detail,
            marker=None):
        """
        Gets billing information for a particular time period for a namespace
        and optionally its buckets. Note that this method will return one and
        only one time sample so if the start_time and end_time do not align
        with a sampled time period, the matching samples will be aggregated.
        Check the response for the actual time period sampled. Also note that
        due to the fact that sampling a namespace's buckets takes some time
        it's possible that the values for the buckets will not sum to equal
        the namespace's values.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        TENANT_ADMIN


        TODO: Add JSON sample

        :param namespace: The namespace containing the bucket
        :param start_time: Starting time for the sample(s) in ISO-8601 minute
        format
        :param end_time: Ending time for the sample(s) in ISO-8601 minute
        format
        :param include_bucket_detail: Whether to include bucket details
        :param marker: Used to continue a truncated response. Omit this
        parameter on the first request
        """
        params = {
            'start_time': start_time,
            'end_time': end_time,
            'include_bucket_detail': include_bucket_detail,
            'marker': marker
        }

        if marker is None:
            return self.conn.get(
                'object/billing/namespace/{0}/sample'.format(namespace),
                params=params)



# Standard lib imports
# None

# Third party imports
# None

# Project level imports
# None


class Health():

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_health(self):
        """
        Perform an HTTP GET against the ViPR endpoint to
        retrieve the VDC's health

        Example JSON result from the API:

        {
            "status": "GOOD",
            "sw_version": "fabric-1.0.0.1",
            "node_count": 48
        }

        :return: The health information
        """
        return self.conn.get('vdc/fabric/health.json')



# Standard lib imports
# None

# Third party imports
# None

# Project level imports
# None


class Namespace():

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_namespaces(self):
        """
        Returns the identifiers for all of the configured namespaces

        Required role(s):

        SYSTEM_MONITOR
        SYSTEM_ADMIN

        Example JSON result from the API:

        {
            "namespace": [
                {
                    "link": {
                        "href": "/object/namespaces/namespace/namespace1",
                        "rel": "self"
                    },
                    "name": "namespace1",
                    "id": "namespace1"
                },
                {
                    "link": {
                        "href": "/object/namespaces/namespace/namespace2",
                        "rel": "self"
                    },
                    "name": "namespace2",
                    "id": "namespace2"
                }
            ]
        }
        """
        return self.conn.get(url='object/namespaces.json')

    def get_namespace_info(self, namespace_identifier):
        """
        Returns detailed info for a specific namespace

        Required role(s):

        SYSTEM_MONITOR
        SYSTEM_ADMIN

        Example JSON result from the API:

        {
            "default_data_services_vpool": "rn:storageos:ReplicationGroupInfo:0c10681a-9b32-4b85-961f-1424ba1e7133:global",
            "remote": "None",
            "name": "namespace1",
            "tags": [],
            "global": "None",
            "disallowed_vpools_list": [],
            "vdc": "None",
            "inactive": "False",
            "link": {
                "href": "/object/namespaces/namespace/namespace1",
                "rel": "self"
            },
            "default_object_project": "rn:storageos:Project:9dec8645-b467-4c6a-a2f2-287cbc746201:global",
            "allowed_vpools_list": [],
            "id": "namespace1",
            "tenant": "rn:storageos:TenantOrg:0324851a-9411-474d-a8f7-29bfff07f536:global"
        }

        :param namespace_identifier: Namespace identifier
        """
        return self.conn.get(
            url='object/namespaces/namespace/{0}.json'.format(
                namespace_identifier))



# Standard lib imports
# None

# Third party imports
# None

# Project level imports
# None


class UserManagement():

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_objectusers(self, namespace=None):
        """
        Get the list of users provisioned for object storage

        Required role(s):

        SYSTEM_MONITOR
        SYSTEM_ADMIN

        Example JSON result from the API:

        {
            "blobuser": [
                {
                    "userid": "user@somewhere",
                    "namespace": "namespace1"
                },
                {
                    "userid": "user2@helloworld",
                    "namespace": "namespace2"
                }
            ]
        }

        If namespace is provided then get list of all the
        users associated with a namespace

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        {
            "blobuser": [
                {
                    "userid": "user@somewhere",
                    "namespace": "namespace1"
                }
            ]
        }

        :param namespace: Example: namespace1

        Required role(s):

        SYSTEM_MONITOR
        SYSTEM_ADMIN


        """
        if namespace:
            return self.conn.get(url='object/users/{0}'.format(namespace))
        else:
            return self.conn.get(url='object/users.json')

    def deactivate_objectuser(self, user, namespace=None):
        """
        Delete a user and all associated secret keys

        Required role(s):

        SYSTEM_ADMIN

        No JSON is returned from ViPR

        :param user: Example: user@somewhere
        :param namespace: Example: namespace1 (optional)
        """
        if namespace:
            payload = {
                "user": user,
                "namespace": namespace
            }
        else:
            payload = {
                "user": user
            }

        return self.conn.post(url='object/users/deactivate.json',
                              json_payload=payload)

    def add_objectuser(self, user, namespace):
        """
        Add a user, which can subsequently be used to create its secret key

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        {
            "link": {
                "href": "/object/user-secret-keys/test1",
                "rel": "self"
            }
        }

        :param user: Example: user@somewhere
        :param user: Example: namespace1
        """

        payload = {
            "user": user,
            "namespace": namespace
        }

        return self.conn.post(url='object/users.json',
                              json_payload=payload)

    def lock_objectuser(self, user, namespace=None):
        """
        Lock a user

        Required role(s):

        SYSTEM_ADMIN
        TENANT_ADMIN

        Example JSON result from the API:

        TODO: JSON data

        :param user: Example: user@somewhere
        :param namespace: Example: namespace1 (optional)
        """
        if namespace:
            payload = {
                "user": user,
                "namespace": namespace
            }
        else:
            payload = {
                "user": user
            }

        return self.conn.put(url='object/users/lock',
                              json_payload=payload)

    def get_objectuser_lock(self, user, namespace=None):
        """
        Get user lock status

        Required role(s):

        SYSTEM_ADMIN
        TENANT_ADMIN

        Example JSON result from the API:

        TODO: JSON data

        :param user: Example: user@somewhere
        :param namespace: Example: namespace1 (optional)
        """

        if namespace:
            return self.conn.get(
                url='object/users/lock/{0}/{1}'.format(user, namespace))
        else:
            return self.conn.get(
                url='object/users/lock/{0}'.format(user))



# Standard lib imports
# None

# Third party imports
# None

# Project level imports
# None


class Bucket():

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_buckets(self, namespace):
        """
        List of buckets associated with the namespace

        Required role(s):

        SYSTEM_ADMIN
        TENANT_ADMIN
        RESTRICTED_SYSTEM_ADMIN

        Example JSON result from the API:

        {
            "object_bucket": [
                {
                    "remote": "None",
                    "name": "bucket1",
                    "tags": [],
                    "global": "None",
                    "vdc": "None",
                    "inactive": "False",
                    "id": "namespace1.bucket1"
                },
                {
                    "remote": "None",
                    "name": "bucket2",
                    "tags": [],
                    "global": "None",
                    "vdc": "None",
                    "inactive": "False",
                    "id": "namespace1.bucket2"
                }
            ]
        }

        List of buckets associated with the namespace

        :param namespace: The namespace to query for buckets
        """
        return self.conn.get(url='object/bucket/{0}'.format(namespace))

    def get_bucket_retention(self, bucket_name):
        """
        Returns default retention period for bucket

        Example JSON result from the API:

        {
            "period": -2
        }

        :param bucket_name: The bucket name to fetch the retention period
        """
        return self.conn.get(
            url='object/bucket/{0}/retention'.format(bucket_name))

    def update_bucket_owner(self, namespace, new_owner, bucket_name):
        """
        Update Owner for a bucket

        Example JSON result from the API:

        There is no response body for this call

        :param namespace: The namespace
        :param new_owner: New Owner must be a valid user
        :param bucket_name: Name of the bucket
        """
        payload = {
            "namespace": namespace,
            "new_owner": new_owner
        }

        return self.conn.post(
            url='object/bucket/{0}/owner'.format(bucket_name),
            json_payload=payload)

    def update_bucket_retention(self, period, bucket_name):
        """
        Sets or updates default retention setting for bucket

        Example JSON result from the API:

        TODO: Add JSON sample

        :param period: Default retention period in seconds
        :param bucket_name: Name of the bucket
        """
        payload = {
            "period": period
        }

        return self.conn.put(
            url='object/bucket/{0}/retention'.format(bucket_name),
            json_payload=payload)

    def create_bucket(self, bucket_name, vpool, namespace, filesystem_enabled,
                      project=None, head_type=None):
        """
        Create a bucket

        Required role(s):

        This call has no restrictions

        Example JSON result from the API:

        {
            "name": "bobsbucket",
            "id": "namespace1.bobsbucket",
            "inactive": "false",
            "global": None,
            "remote": None,
            "vdc": None,
            "tags": []
        }

        :param bucket_name: Name of the bucket
        :param project: Project ID for the bucket
        :param vpool: vpool ID for the bucket
        :param filesystem_enabled: flag indicating whether file-system is
            enabled for this bucket
        :param head_type: HeadType indicates the object head type that is
            allowed to access the bucket. If the bucket has FS-Enabled,
            then the FS heads are implicitly allowed to access this bucket 
        :param namespace: namespace associated with the user/tenant that is
            allowed to access the bucket    
        """

        payload = {
            "name": bucket_name,
            "project": project,
            "vpool": vpool,
            "filesystem_enabled": filesystem_enabled,
            "head_type": head_type,
            "namespace": namespace
        }

        return self.conn.post(url='object/bucket',
                              json_payload=payload)

    def get_bucket_lock(self, bucket_name, namespace=None):
        """
        Get bucket lock status

        Required role(s):

        SYSTEM_ADMIN
        TENANT_ADMIN
        RESTRICTED_SYSTEM_ADMIN

        Example JSON result from the API:

        TODO: Add JSON sample

        :param bucket_name: Example: bobsbucket
        :param namespace: Example: namespace1 (optional)
        """

        if namespace:
            return self.conn.get(
                url='object/bucket/{0}/{1}/lock'.format(
                    bucket_name, namespace))
        else:
            return self.conn.get(
                url='object/bucket/{0}/lock'.format(bucket_name))

    def lock_bucket(self, bucket_name, is_locked='true', namespace=None):
        """
        Locks the Bucket.
        Once is_locked is set to true no operation
        can be performed on the bucket.

        Required role(s):

        SYSTEM_ADMIN
        TENANT_ADMIN
        RESTRICTED_SYSTEM_ADMIN

        Example JSON result from the API:

        TODO: Add JSON sample

        :param bucket_name: Example: bobsbucket
        :param lock: Example: 'true' or 'false'
        :param namespace: Example: namespace1 (optional)
        """

        payload = {}

        if namespace:
            payload = {
                "namespace": namespace
            }

        return self.conn.put(
            url='object/bucket/{0}/lock/{1}'.format(bucket_name, is_locked),
            json_payload=payload
        )



# Standard lib imports
# None

# Third party imports
# None

# Project level imports
# None


class HealthMonitor():

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_stats(self, node_id=None, interval=0):
        """
        Perform an HTTP GET against the ViPR endpoint to
        retrieve statistics of a virtual machine and its
        active services

        If an interval value is passed it will return differential
        disk stats: difference between first report (contains stats
        for the time since system startup) and second report
        (stats collected during the interval since the first report)

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR


        Example JSON result from the API:

        Too large to show


        :param node_id: node ids for which stats are collected
        :param interval: Specifies amount of time in seconds for differential
        stats

        :return: Virtual machine stats including memory
        usage, I/O for each device and load average numbers.
        Service stats include service memory usage, command
        that invoked it, file descriptors count and other
        stats (uptime, start time, thread count)
        """
        params = {
            'node_id': node_id,
            'interval': interval
        }

        return self.conn.get(
            'monitor/stats.json', params=params)

    def get_health(self, node_id=None):
        """
        Perform an HTTP GET against the ViPR endpoint to
        retrieve health of node and its services

        Service health status: Good - when a service is up and running
        Unavailable - when a service is not running but is
        registered in coordinator
        Restarted - when service is restarting

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR


        Example JSON result from the API:

        {
            "node_health_list": {
                "node_health": [
                    {
                        "status": "Good",
                        "ip": "172.29.112.104",
                        "node_id": "nilea01-r08-06",
                        "service_health_list": {
                            "service_health": [
                                {
                                    "status": "Good",
                                    "name": "georeplayer"
                                },
                                {
                                    "status": "Good",
                                    "name": "fabric_zk_agent"
                                },
                                {
                                    "status": "Good",
                                    "name": "cm"
                                },
                                {
                                    "status": "Good",
                                    "name": "rm"
                                },
                                {
                                    "status": "Good",
                                    "name": "sysagent"
                                },
                                {
                                    "status": "Good",
                                    "name": "casheadsvc"
                                },
                                {
                                    "status": "Good",
                                    "name": "hdfssvc"
                                },
                                {
                                    "status": "Good",
                                    "name": "ssm"
                                },
                                {
                                    "status": "Good",
                                    "name": "fabricapi"
                                },
                                {
                                    "status": "Good",
                                    "name": "blobsvc"
                                },
                                {
                                    "status": "Good",
                                    "name": "georeceiver"
                                }
                            ]
                        }
                    },
                    {
                        "status": "Good",
                        "ip": "172.29.112.79",
                        "node_id": "nilea01-r05-05",
                        "service_health_list": {
                            "service_health": [
                                {
                                    "status": "Good",
                                    "name": "georeplayer"
                                },
                                {
                                    "status": "Good",
                                    "name": "fabric_zk_agent"
                                },
                                {
                                    "status": "Good",
                                    "name": "cm"
                                },
                                {
                                    "status": "Good",
                                    "name": "rm"
                                },
                                {
                                    "status": "Good",
                                    "name": "sysagent"
                                },
                                {
                                    "status": "Good",
                                    "name": "casheadsvc"
                                },
                                {
                                    "status": "Good",
                                    "name": "hdfssvc"
                                },
                                {
                                    "status": "Good",
                                    "name": "ssm"
                                },
                                {
                                    "status": "Good",
                                    "name": "fabricapi"
                                },
                                {
                                    "status": "Good",
                                    "name": "blobsvc"
                                },
                                {
                                    "status": "Good",
                                    "name": "georeceiver"
                                }
                            ]
                        }
                    }
                ]
            }
        }


        :param node_id: Node ID for which health stats are collected

        :return: The health of the node and its services
        """
        params = {
            'node_id': node_id
        }

        return self.conn.get(
            'monitor/health.json', params=params)

    def get_diagnostics(self, node_id=None, verbose=None):
        """
        Get results of diagtool shell script for all virtual
        machines in a ViPR controller appliance. Also gives
        test details when verbose option is set.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            "node_diagnostics_list": {
                "node_diagnostics": {
                    "ip": "199.34.96.70",
                    "tests": "None",
                    "node_id": "nilea01-r05-05"
                }
            }
        }

        :param node_id: Node IDs for which diagnostic results are collected
        :param verbose: When set to "1" it will run the command with -v option
        """
        params = {
            'node_id': node_id,
            'verbose': verbose
        }

        return self.conn.get(
            'monitor/diagnostics.json', params=params)

    def get_storage_stats(self):
        """
        Get the current capacity for object, file and block storage.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            "unstructured": {
                "capacity_kb": "1.103508799488E13"
            },
            "controller": {
                "file_managed_capacity_kb": "0.0",
                "block_managed_capacity_kb": "0.0",
                "free_managed_capacity_kb": "0.0"
            },
            "object": {
                "capacity_kb": "1.103508799488E13"
            }
        }
        """
        return self.conn.get('monitor/storage.json')



# Standard lib imports
import os

# Third party imports
import requests

# Project level imports
from viperpy.util.exceptions import ViperpyException


class TokenRequest(object):
    """
    This is a helper class to fetch a new token from the ViPR controller
    and return the token as well as store it locally. Prior to fetching a new
    token we check if we have a local token and if so, whether or not it is
    still valid
    """

    def __init__(self, username, password, vipr_endpoint, token_endpoint,
                 verify_ssl, token_filename, token_location, request_timeout,
                 cache_token):
        """
        Create a new TokenRequest instance
        """
        self.username = username
        self.password = password
        self.vipr_endpoint = vipr_endpoint
        self.token_endpoint = token_endpoint
        self.verify_ssl = verify_ssl
        self.token_verification_endpoint = vipr_endpoint + '/user/whoami'
        self.token_filename = token_filename
        self.token_location = token_location
        self.request_timeout = request_timeout
        self.cache_token = cache_token
        self.token_file = os.path.join(
            self.token_location, self.token_filename)
        self.session = requests.Session()

    def get_new_token(self):
        """
        Request a new authentication token from ViPR and persist it
        to a file for future usage if cache_token is true

        :return: Returns a valid token, or None if failed
        """
        self.session.auth = (self.username, self.password)

        req = self.session.get(self.token_endpoint,
                               verify=self.verify_ssl,
                               headers={'Accept': 'application/json'},
                               timeout=self.request_timeout)

        if req.status_code == 401:
            raise ViperpyException(
                http_status_code=req.status_code,
                message='Invalid username or password used')
        if req.status_code != 200:
            raise ViperpyException(
                http_status_code=req.status_code)

        token = req.headers['x-sds-auth-token']

        if self.cache_token:
            with open(self.token_file, 'w') as token_file:
                token_file.write(token)

        return token

    def get_token(self):
        """
        Attempt to get an existing token, if successful then ensure it
        hasn't expired yet. If its expired, fetch a new token

        :return: A token
        """
        token = self._get_existing_token()

        if token:
            req = self._request(token, self.token_verification_endpoint)

            if req.status_code == requests.codes.ok:
                return token

        return self.get_new_token()

    def _get_existing_token(self):
        """
        Attempt to open and read the token file if it exists

        :return: If available return the token, if not return None
        """
        if os.path.isfile(self.token_file):
            with open(self.token_file, 'r') as token_file:
                return token_file.read()
        return None

    def _request(self, token, url):
        """
        Perform a request and place the token header inside the request header

        :param token: A valid token
        :param url:  The URL to perform a request
        :return: Request Object
        """
        headers = {'Accept': 'application/json',
                   'X-SDS-AUTH-TOKEN': token}
        try:
            return self.session.get(url, verify=self.verify_ssl,
                                    headers=headers,
                                    timeout=self.request_timeout)
        except requests.ConnectionError as conn_err:
            raise ViperpyException(message=conn_err.message)
        except requests.HTTPError as http_err:
            raise ViperpyException(message=http_err.message)
        except requests.RequestException as req_err:
            raise ViperpyException(message=req_err.message)



# Standard lib imports
# None

# Third party imports
# None

# Project level imports
# None


class Node():

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_nodes(self, maximum=-1, index=0):
        """
        Perform an HTTP GET against the ViPR endpoint to
        retrieve a list of nodes in the array

        Example JSON result from the API:

        {
            "node": [
                {
                    "link": {
                        "href": "/vdc/fabric/nodes/urn:fabric:node:02edcf91-5086-11e3-a7f8-001e6769f9a1:",
                        "rel": "self"
                    },
                    "id": "urn:fabric:node:02edcf91-5086-11e3-a7f8-001e6769f9a1:",
                    "name": "nilea01-r05-02"
                },
                {
                    "link": {
                        "href": "/vdc/fabric/nodes/urn:fabric:node:24115e71-4fbe-11e3-b044-001e6769e808:",
                        "rel": "self"
                    },
                    "id": "urn:fabric:node:24115e71-4fbe-11e3-b044-001e6769e808:",
                    "name": "nilea01-r05-04"
                }
            ]
        }

        :param maximum: Maximum number of items to return or -1 for all
        :param index: Index to start

        :return: Returns list of nodes in the array
        """
        params = {
            'max': maximum,
            'index': index
        }

        return self.conn.get('vdc/fabric/nodes', params=params)



# Standard lib imports
# None

# Third party imports
# None

# Project level imports
# None


class Tenants():

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_tenants_bulk(self):
        """
        Perform an HTTP GET against the ViPR endpoint to
        retrieve the tenants bulk payload

        :return: A list of Tenant IDs
        """
        json_data = self.conn.get('tenants/bulk.json')
        return json_data['id']

    def get_tenant(self, tenant_id):
        """
        Perform an HTTP GET against the ViPR endpoint to
        retrieve the tenant's information

        :return: The tenant's information
        """
        return self.conn.get('tenants/{0}.json'.format(tenant_id))

    def get_subtenants(self, tenant_id):
        return self.conn.get('tenants/{0}/subtenants.json'.format(tenant_id))



# Standard lib imports
# None

# Third party imports
# None

# Project level imports
# None


class UserSecretKey():

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_user_secret_keys(self, uid, namespace=None):
        """
        Get all secret keys for specific user

        Required role(s):

        SYSTEM_MONITOR
        SYSTEM_ADMIN

        Example JSON result from the API:

        {
            "secret_key_1": "NfgLnEzxKV0+iL5i0cfYwDnD7310uQC2VLhMsyVx",
            "key_timestamp_2": "2014-11-03 22:31:53.103",
            "key_timestamp_1": "2014-11-03 22:31:48.432",
            "link": {
                "href": "/object/secret-keys",
                "rel": "self"
            },
            "secret_key_2": "bAt/D2Ybc1Z+iYghwxTk6hIab4u4Mn5fWY/Iww1Z2"
        }

        :param uid: Valid user identifier to get the keys from
        :param namespace: The namespace
        """

        if namespace:
            return self.conn.get(
                url='object/user-secret-keys/{0}/{1}'.format(uid, namespace))
        else:
            return self.conn.get(
                url='object/user-secret-keys/{0}'.format(uid))

    def create_new_secret_key(self, uid, namespace=None,
                              key_expiration=2592000):
        """
        Create new key for a specific user

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        {
            "link": {
                "rel": "self",
                "href":"/object/user-secret-keys/joedeleteme"
            },
            "secret_key": "p3PZyb//Ch6tM0fUsnesYYnGb+6JHV8WHzS5YHjg",
            "key_timestamp": "2014-12-24 02:08:40.181"
        }

        :param uid: Valid user identifier to create a key for
        :param namespace: The namespace
        :param key_expiration: Defaults to 30 days (2592000 seconds)
        """
        payload = {
            "existing_key_expiry_time_mins": key_expiration,
            "namespace": namespace
        }

        return self.conn.post(url='object/user-secret-keys/{0}'.format(uid),
                              json_payload=payload)

    def deactivate_user_secret_key(self, uid, namespace=None,
                                   secret_key=None):
        """
        Create new key for a specific user

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        :param uid: Valid user identifier to get delete the keys from
        :param namespace: The namespace
        :param secret_key: The secret key to deactivate
        """
        payload = {
            "secret_key": secret_key,
            "namespace": namespace
        }

        return self.conn.post(
            url='object/user-secret-keys/{0}/deactivate'.format(uid),
            json_payload=payload)



# Standard lib imports
# None

# Third party imports
# None

# Project level imports
# None


class Upgrade():

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_target_version(self):
        """
        Show the current target version

        Required role(s):

        SYSTEM_ADMIN
        RESTRICTED_SYSTEM_ADMIN

        Example JSON result from the API:

        {
            "target_version": "vipr-2.1.0.1.447"
        }

        """
        return self.conn.get('upgrade/target-version.json')

    def get_cluster_state(self, force=False):
        """
        Show cluster state

        Required role(s):

        SYSTEM_ADMIN
        SECURITY_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            "target_state": {
                "available_versions": {
                    "available_version": "vipr-2.1.0.1.447"
                },
                "current_version": "vipr-2.1.0.1.447",
                "config_version": "1416366397891"
            },
            "control_nodes": {
                "entry": [
                    {
                        "value": {
                            "available_versions": {
                                "available_version": "vipr-2.1.0.1.447"
                            },
                            "current_version": "vipr-2.1.0.1.447",
                            "config_version": "1416366397891"
                        },
                        "key": "vipr3"
                    },
                    {
                        "value": {
                            "available_versions": {
                                "available_version": "vipr-2.1.0.1.447"
                            },
                            "current_version": "vipr-2.1.0.1.447",
                            "config_version": "1416366397891"
                        },
                        "key": "vipr2"
                    },
                    {
                        "value": {
                            "available_versions": {
                                "available_version": "vipr-2.1.0.1.447"
                            },
                            "current_version": "vipr-2.1.0.1.447",
                            "config_version": "1416366397891"
                        },
                        "key": "vipr1"
                    }
                ]
            },
            "cluster_state": "STABLE"
        }

        :param force: If force=True it will show all removable versions even
        though the installed versions are less than MAX_SOFTWARE_VERSIONS
        """
        params = {
            'force': force
        }

        return self.conn.get(
            'upgrade/cluster-state.json', params=params)

    def get_download_progress(self):
        """
        Check the version downloading progress.
        The download could be from remote repository

        Required role(s):

        SYSTEM_ADMIN
        RESTRICTED_SYSTEM_ADMIN

        Example JSON result from the API:

        {
            "progress": "None",
            "imageSize": "0"
        }

        """
        return self.conn.get('upgrade/image/download/progress.json')



# Standard lib imports
# None

# Third party imports
# None

# Project level imports
# None


class Configuration():

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_config_properties(self):
        """
        Get system configuration properties

        Required role(s):

        SYSTEM_ADMIN
        SECURITY_ADMIN

        Example JSON result from the API:

        (Output is much longer, shortened for space concerns)

        {
            "properties": {
                "entry": [
                    {
                        "value": "0",
                        "key": "certificate_version"
                    },
                    {
                        "value": "5.9",
                        "key": "compute_redhat_linux_version"
                    },
                    {
                        "value": "11",
                        "key": "compute_suse_linux_version"
                    },
                    {
                        "value": "5.0",
                        "key": "compute_vmware_vcenter_version"
                    },
                    {
                        "value": "6.0.6002",
                        "key": "compute_windows_version"
                    }
                ]
            }
        }
        """
        return self.conn.get('config/properties.json')



# Standard lib imports
# None

# Third party imports
# None

# Project level imports
# None


class Services():

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_services(self):
        """
        Perform an HTTP GET against the ViPR endpoint to
        retrieve a list of services available on the array

        Example JSON result from the API:

        {
            "service": [
                {
                    "link": {
                        "href": "/vdc/fabric/services/urn:fabric:service:Block:",
                        "rel": "self"
                    },
                    "id": "urn:fabric:service:Block:",
                    "name": "Block"
                },
                {
                    "link": {
                        "href": "/vdc/fabric/services/urn:fabric:service:Object:",
                        "rel": "self"
                    },
                    "id": "urn:fabric:service:Object:",
                    "name": "Object"
                }
            ]
        }

        :return: Returns a list of services available on the array
        """
        return self.conn.get('vdc/fabric/services')



# Standard lib imports
# None

# Third party imports
# None

# Project level imports
# None


class Capacity():

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_capacity(self):
        """
        Perform an HTTP GET against the ViPR endpoint to
        retrieve the aggregated capacity info

        Example JSON result from the API:

        {
            "total_capacity": 11311005
        }

        :return: Capacity info for the entire Fabric
        """
        return self.conn.get('vdc/fabric/capacity')




"""-*- coding: utf-8 -*-
ViperPy Setup
"""
try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages


def read(relative):
    """
    Read file contents and return a list of lines.
    ie, read the VERSION file
    """
    contents = open(relative, 'r').read()
    return [l for l in contents.split('\n') if l != '']


setup(
    name='viperpy',
    url='https://github.com/chadlung/viperpy',
    keywords=['vipr'],
    version=read('VERSION')[0],
    description='A library for interacting with the EMC ViPR API',
    author='Chad Lung, EMC Rubicon',
    author_email='chad.lung@gmail.com',
    tests_require=read('./test-requirements.txt'),
    install_requires=read('./requirements.txt'),
    test_suite='nose.collector',
    zip_safe=False,
    include_package_data=True,
    packages=find_packages(exclude=['ez_setup'])
)



# Standard lib imports
# None

# Third party imports
# None

# Project level imports
# None


class Disk():

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_disks(self, maximum=-1, index=0):
        """
        Perform an HTTP GET against the ViPR endpoint to
        retrieve the list of disks in the array

        Example JSON result from the API:

        {
            "disk": [
                {
                    "node_id": "urn:fabric:node:8f2c5a01-5551-11e3-b4fd-001e676a04b4:",
                    "link": {
                        "href": "/vdc/fabric/disks/urn:fabric:disk:003e12d7-9b4d-41fc-a353-ceeadec26827:",
                        "rel": "self"
                    },
                    "id": "urn:fabric:disk:003e12d7-9bdd-42fc-a353-ceeadec26827:",
                    "name": "/dev/sdav1"
                },
                {
                    "node_id": "urn:fabric:node:37f71ce1-51f8-11e3-b5f5-001e6769fd3e:",
                    "link": {
                        "href": "/vdc/fabric/disks/urn:fabric:disk:0391c6a3-35ed-449e-a247-4920c375c226:",
                        "rel": "self"
                    },
                    "id": "urn:fabric:disk:0091b6a3-35ed-449e-a247-4920c375c276:",
                    "name": "/dev/sdae1"
                }
            ]
        }

        :param maximum: Maximum number of items to return or -1 for all
        :param index: Index to start

        :return: Returns list of disks in the array
        """
        params = {
            'index': index,
            'max': maximum
        }

        return self.conn.get('vdc/fabric/disks', params=params)

    def get_disk_by_urn(self, urn=''):
        """
        Perform an HTTP GET against the ViPR endpoint to
        retrieve info about a single disk

        Example JSON result from the API:

        {
            "disk_hwid": "1EG5TP7B",
            "link": {
                "href": "/vdc/fabric/disks/urn:fabric:disk:0091c6a3-35fd-449e-a247-4920c375c226:",
                "rel": "self"
            },
            "id": "urn:fabric:disk:0091c6a3-35ed-449e-a247-4920c335c226:",
            "disk_swid": "0091c6a3-37ed-449e-a247-4920c375c226",
            "name": "/dev/sdae1"
        }

        :param urn: URN value of the disk to fetch info on

        :return: Returns info about a single disk
        """
        return self.conn.get('vdc/fabric/disks/{0}'.format(urn))

    def get_disk_capacity(self, urn=''):
        """
        Perform an HTTP GET against the ViPR endpoint to
        retrieve single disk capacity info

        Example JSON result from the API:

        {
            "total_capacity": 6021
        }

        :param urn: URN value of the disk to fetch capacity info on

        :return: Returns single disk capacity info
        """
        return self.conn.get('vdc/fabric/disks/{0}/capacity'.format(urn))



# Standard lib imports
import json
import os

# Third party imports
import requests

# Project level imports
from viperpy.api.fabric.capacity import Capacity as FabricCapacity
from viperpy.api.fabric.disk import Disk
from viperpy.api.fabric.health import Health
from viperpy.api.fabric.node import Node
from viperpy.api.fabric.services import Services

from viperpy.api.object_data_control.billing import Billing
from viperpy.api.object_data_control.bucket import Bucket
from viperpy.api.object_data_control.capacity_info \
    import Capacity as ObjectDataControlCapacity
from viperpy.api.object_data_control.namespace import Namespace
from viperpy.api.object_data_control.object_soft_quota import ObjectSoftQuota
from viperpy.api.object_data_control.user_mangement import UserManagement
from viperpy.api.object_data_control.user_secret_key import UserSecretKey

from viperpy.api.other.audit import Audit
from viperpy.api.other.user_info import UserInfo

from viperpy.api.system_management.configuration import Configuration
from viperpy.api.system_management.health_monitor import HealthMonitor
from viperpy.api.system_management.upgrade import Upgrade

from viperpy.api.tenant.tenants import Tenants

from viperpy.util.exceptions import ViperpyException
from viperpy.util.token_request import TokenRequest


class Viperpy(object):

    def __init__(self, username=None, password=None, token=None,
                 vipr_endpoint=None, token_endpoint=None, verify_ssl=False,
                 token_filename='viperpy.tkn', token_location='/tmp',
                 request_timeout=15.0, cache_token=True):
        """
        Creates the ViperPy class that the client will directly work with

        :param username: The username to fetch a token
        :param password: The password to fetch a token
        :param token: Supply a valid token to use instead of username/password
        :param vipr_endpoint: The URL where ViPR is located
        :param token_endpoint: The URL where the ViPR login is located
        :param verify_ssl: Verify SSL certificates
        :param token_filename: The name of the cached token filename
        :param token_location: By default this is stored in /tmp
        :param request_timeout: How long to wait for ViPR to respond
        :param cache_token: Whether to cache the token, by default this is true
        you should only switch this to false when you want to directly fetch
        a token for a user
        """

        self.username = username
        self.password = password
        self.token = token
        self.vipr_endpoint = vipr_endpoint.rstrip('/')
        self.token_endpoint = token_endpoint.rstrip('/')
        self.verify_ssl = verify_ssl
        self.token_filename = token_filename
        self.token_location = token_location
        self.request_timeout = request_timeout
        self.cache_token = cache_token
        self._session = requests.Session()
        self._token_request = TokenRequest(
            username=self.username,
            password=self.password,
            vipr_endpoint=self.vipr_endpoint,
            token_endpoint=self.token_endpoint,
            verify_ssl=self.verify_ssl,
            token_filename=self.token_filename,
            token_location=self.token_location,
            request_timeout=self.request_timeout,
            cache_token=self.cache_token)
        self.token_file = os.path.join(
            self.token_location, self.token_filename)

        # API -> Fabric
        self.fabric_capacity = FabricCapacity(self)
        self.disk = Disk(self)
        self.health = Health(self)
        self.node = Node(self)
        self.services = Services(self)

        # API -> Object Data Control
        self.billing = Billing(self)
        self.bucket = Bucket(self)
        self.object_data_control_capacity = ObjectDataControlCapacity(self)
        self.namespace = Namespace(self)
        self.user_management = UserManagement(self)
        self.user_secret_key = UserSecretKey(self)
        self.object_soft_quota = ObjectSoftQuota(self)

        # API -> Other
        self.audit = Audit(self)
        self.user_info = UserInfo(self)

        # API -> System Management
        self.configuration = Configuration(self)
        self.health_monitor = HealthMonitor(self)
        self.upgrade = Upgrade(self)

        # API -> Tenant
        self.tenants = Tenants(self)

    def get_token(self):
        """
        Get a token directly back, typically you want to set the cache_token
        param for ViperPy to false for this call.

        :return: A valid token or an ViperPy exception
        """
        return self._token_request.get_new_token()

    def remove_cached_token(self):
        """
        Remove the cached token file, this is useful if you switch users
        and want to use a different token
        """
        if os.path.isfile(self.token_file):
            os.remove(self.token_file)

    def _fetch_headers(self):
        if self.token:
            return {'Accept': 'application/json',
                    'Content-Type': 'application/json',
                    'x-sds-auth-token': self.token}
        else:
            return {'Accept': 'application/json',
                    'Content-Type': 'application/json',
                    'x-sds-auth-token': self._token_request.get_token()}

    def _construct_url(self, url):
        return '{0}/{1}'.format(self.vipr_endpoint, url)

    def get(self, url, params=None):
        return self._request(url, params=params)

    def post(self, url, json_payload='{}'):
        return self._request(url, json_payload, http_verb='POST')

    def put(self, url, json_payload='{}'):
        return self._request(url, json_payload, http_verb='PUT')

    def _request(self, url, json_payload='{}', http_verb='GET', params=None):
        json_payload = json.dumps(json_payload)

        try:
            if http_verb == "PUT":
                req = self._session.put(
                    self._construct_url(url),
                    verify=self.verify_ssl,
                    headers=self._fetch_headers(),
                    timeout=self.request_timeout,
                    data=json_payload)
            elif http_verb == 'POST':
                req = self._session.post(
                    self._construct_url(url),
                    verify=self.verify_ssl,
                    headers=self._fetch_headers(),
                    timeout=self.request_timeout,
                    data=json_payload)
            else:  # Default to GET
                req = self._session.get(
                    self._construct_url(url),
                    verify=self.verify_ssl,
                    headers=self._fetch_headers(),
                    timeout=self.request_timeout,
                    params=params)

            if req.status_code != 200:
                raise ViperpyException(
                    http_status_code=req.status_code,
                    vipr_message=req.text)
            return req.json()

        except requests.ConnectionError as conn_err:
            raise ViperpyException(message=conn_err.message)
        except requests.HTTPError as http_err:
            raise ViperpyException(message=http_err.message)
        except requests.RequestException as req_err:
            raise ViperpyException(message=req_err.message)
        except ValueError:
            return
